package editor;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * SQLite database handler. Establishes a connection with the database, channels the queries 
 * through a single statement object and disconnects from the database.
 * 
 * @author hqcasanova
 *  
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class ConnectDB {
	private Connection conn;
	private Statement stmt;
	
	/**
     * Customised constructor. Connects to the given SQLITE database, checking that the connection 
     * could be established. If not, the program is aborted.
     * @param sDbPath Path to the database file     
     */
	public ConnectDB (String sDbPath) {
		conn = null;
		stmt = null;

		try {
			Class.forName("org.sqlite.JDBC").newInstance(); 
			conn = DriverManager.getConnection ("jdbc:sqlite:" +sDbPath);
			stmt = conn.createStatement();
		} catch (SQLException error) {
		        System.out.println ("Program aborted. Connection to the database could "+
		      		                "not be stablished: " +error);
		        System.exit (0);
	    } catch (Exception error) {
		        System.out.println ("Program aborted. Could not access the data source: " +error);
		        System.exit (0);
	    }		
	}
	
	/**
	 * Retrieves the "statement handler" for the established database connection.
	 * @return Statement handler
	 */
	public Statement getStatement() throws SQLException {
		return stmt;
	}
	
	/**
	 * Disposes of all database-related resources.
	 */
    public void closeDataSource () throws SQLException {
	    stmt.close ();
	    conn.close (); 
	}
}
