package rtf;

import java.awt.Color;

/**
 * 
 * @autor mtordesillas
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class ChunkFormat {
	
	private Boolean bold;
	private Boolean underlined;
	private Boolean italic;
	private Color color;
	private int fontSize;
	private String font;
	
	public ChunkFormat(){
		this.bold = false;
		this.underlined = false;
		this.italic = false;
		this.color = Color.BLACK;
		this.fontSize = 14;
		this.font = "Sarakanda";
	}
	
	public ChunkFormat(String fontName, int fontSize, Color fontColor){
		this.bold = false;
		this.underlined = false;
		this.italic = false;
		this.color = fontColor;
		this.fontSize = TranslateHTMLSize(fontSize);
		this.font = fontName;
	}
	
	public ChunkFormat(ChunkFormat format){
		this.bold = format.bold;
		this.underlined = format.underlined;
		this.italic = format.italic;
		this.color = format.color;
		this.fontSize = format.fontSize;
		this.font = format.font;
	}
	
	public Boolean IsChunkBold(){
		return this.bold;
	}
	
	public Boolean IsChunkItalic(){
		return this.italic;
	}
	
	public Boolean IsChunkUnderlined(){
		return this.underlined;
	}
	
	public Color GetColor(){
		return this.color ;
	}
	
	public int GetFontSize(){
		return fontSize;
	}
	
	public String GetFontName(){
		return font;
	}
	
	public void SetChunkBold(Boolean bold){
		this.bold = bold;
	}
	
	public void SetChunkItalic(Boolean italic){
		this.italic = italic;
	}
	
	public void SetChunkUnderlined(Boolean underlined){
		this.underlined = underlined;
	}
	
	public void SetChunkFont(String font){
		this.font = font;
	}
	
	public void SetChunkFontSize(int htmlSize){
		this.fontSize = TranslateHTMLSize(htmlSize);
	}
	
	public void SetChunkFontColor(String color){
		this.color = Color.decode(color);
	}

	private int TranslateHTMLSize(int htmlSize){
		int pixelSize;
		
		switch (htmlSize){
		case 1:
			pixelSize = 8;
			break;
		case 2:
			pixelSize = 10;
			break;
		case 3:
			pixelSize = 12;
			break;
		case 4:
			pixelSize = 14;
			break;
		case 5:
			pixelSize = 18;
			break;
		case 6:
			pixelSize = 24;
			break;
		case 7:
			pixelSize = 36;
			break;
		default:
			pixelSize = 14;
			break;
		}
		return pixelSize;
	}

}
