package utils;

import java.awt.Color;
import java.awt.Image;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 *  
 * @author jwdd
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class DataUsers
{
	/**
	 * Número de usuarios
	 * 
	 * @return
	 */
	public static int countUsers()
	{
		Statement stat = null;
		ResultSet rs = null;
		int v = 0;

		String sql = "SELECT COUNT(ID) AS NUM FROM USERS WHERE ID>1";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			v = Integer.valueOf(rs.getString("NUM"));
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return v;
	}

	/**
	 * Número de usuarios activos
	 * 
	 * @return
	 */
	public static int countActiveUsers()
	{
		Statement stat = null;
		ResultSet rs = null;
		int v = 0;

		String sql = "SELECT COUNT(ID) AS NUM FROM USERS WHERE ENABLE=1";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			v = Integer.valueOf(rs.getString("NUM"));
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return v;
	}

	/**
	 * Crea una carpeta
	 * 
	 * @param ruta
	 */

	public static void createFolder(String dir)
	{
		File f = new File(dir);
		try
		{
			f.mkdir();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Borra un directorio
	 * 
	 * @param dir
	 *            ruta
	 * @return
	 */
	public static boolean deleteDirectory(File dir)
	{
		if(dir.isDirectory())
		{
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++)
			{
				boolean success = deleteDirectory(new File(dir, children[i]));
				if(!success)
				{
					return false;
				}
			}
		}

		// The directory is now empty so delete it
		return dir.delete();
	}

	/**
	 * Genera el MD5 de un texto
	 * 
	 * @param text
	 *            el texto
	 * @return el MD5 del texto
	 */
	public static String MD5(String text)
	{
		MessageDigest md;
		try
		{
			md = MessageDigest.getInstance("MD5");
			byte[] md5hash = new byte[32];
			md.update(text.getBytes("UTF-8"), 0, text.length());
			md5hash = md.digest();
			return convertToHex(md5hash);
		}
		catch (Exception e)
		{
			System.out.println(e);
		}

		return null;
	}

	/**
	 * Genera un fichero comprimido con toda la información necesaria para replicar un usuario
	 * 
	 * @param iduser
	 *            id del usuario
	 * @param dpath
	 *            ruta donde se generará el fichero
	 */
	public static void exportProfile(String iduser, String dpath)
	{
		// String spath = System.getProperty("user.dir");
		String opath = Global.USERS_FOLDER + File.separator + iduser;

		NZipCompresser c = new NZipCompresser(new File(opath), dpath);
		try
		{
			c.compress(NZipCompresser.ALLFILES);
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}

	/**
	 * 
	 * Crea un usuario a partir de un fichero de exportacion
	 * 
	 * @param rutal
	 *            al fichero
	 */
	public static void importProfile(String opath)
	{
		String dpath = Global.TEMP_FOLDER + File.separator + "install";

		int n = 2;

		DataUsers.deleteDirectory(new File(dpath)); // Borro la carpeta temporal
		utils.UnZip.unZipIt(opath, dpath); // Descomprimo el archivo con los ficheros del usuario
		String name = readUserName(dpath); // Leo el nombre del usuario
		String name2 = name;
		while (existUserName(name2))
		{
			name2 = name + "_" + n;
			n = n + 1;
		}

		int id = newUserDB(name2); // Inserto el nuevo usuario en la BD y obtengo su id

		String fd = Global.USERS_FOLDER + File.separator + String.valueOf(id); // El profile del nuevo usuario

		new File(fd).mkdir(); // Creo la carpeta del usuario
		importAllDocs(dpath, String.valueOf(id)); // Importo los documentos del editor
		copyDirectory(new File(dpath), new File(fd)); // Copio el resto de ficheros y directorios

		new File(dpath).delete(); // Borra el directorio temporal

		String tcv1 = Global.USERS_FOLDER + File.separator + String.valueOf(id) + File.separator + Global.TCV1_FOLDER;
		String tcv2 = Global.USERS_FOLDER + File.separator + String.valueOf(id) + File.separator + Global.TCV2_FOLDER;

		new File(tcv1).mkdir(); // Por si no existieran las carpetas
		new File(tcv2).mkdir();
	}

	/**
	 * Importa todos los documentos de un usuario a otro
	 * 
	 * @param tempfolder
	 *            Directorio donde se encuentra la información para la exportación
	 * @param newuserid
	 *            Id del usuario
	 */
	public static void importAllDocs(String tempfolder, String newuserid)
	{
		String pathFile = tempfolder + File.separator + Global.USER_IMPORT_DOC_LIST; // Listado con todos los ficheros
		String line = null;
		File ar = new File(pathFile);
		FileReader fr;

		try
		{
			fr = new FileReader(ar);
			BufferedReader br = new BufferedReader(fr);
			while ((line = br.readLine()) != null)
			{
				String[] data = line.split("\\|");
				data[0] = data[0].trim(); // Id antiguo del doc
				data[1] = data[1].trim();
				String filepath = tempfolder + File.separator + data[0] + "." + Global.DOC_EXT;
				String title = data[1];
				insertDocumentFileAndDB(filepath, title, newuserid);
				new File(filepath).delete(); // Borro el fichero que acabo de copiar
			}
			br.close();
			fr.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static String convertToHex(byte[] data)
	{
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < data.length; i++)
		{
			int halfbyte = (data[i] >>> 4) & 0x0F;
			int two_halfs = 0;
			do
			{
				if((0 <= halfbyte) && (halfbyte <= 9))
				{
					buf.append((char) ('0' + halfbyte));
				}
				else
				{
					buf.append((char) ('a' + (halfbyte - 10)));
				}
				halfbyte = data[i] & 0x0F;
			}
			while (two_halfs++ < 1);
		}
		return buf.toString();
	}

	/**
	 * Comprueba si la contraseña de administrador en valida
	 * 
	 * @param pass
	 *            contraseña
	 * @return devuelve true o falsa si la contraseña es correcta o no
	 */
	public static boolean authAdmin(String pass)
	{
		Statement stat = null;
		ResultSet rs = null;
		pass = MD5(pass); // Obtengo el MD5 del pass dado por el usuario

		String sql = "SELECT PASSUSER FROM USERS WHERE ID=1";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			String passdb = rs.getString("PASSUSER"); // Hash guardado en la BD
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
			if(passdb.equals(pass)) return true; // Si es igual
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return false; // Si no es igual
	}

	/**
	 * Me devuelve un Resultset con los usuarios de la aplicación
	 * 
	 * @return
	 */
	public static ResultSet userList()
	{
		Statement stat = null;
		ResultSet rs = null;

		String sql = "SELECT ID,NAMEUSER,ENABLE FROM USERS WHERE ID>1 ORDER BY ID";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			return rs;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	public static ResultSet getUserListWithDocs()
	{
		Statement stat = null;
		ResultSet rs = null;

		String sql = "	SELECT * FROM USERS WHERE EXISTS (SELECT * FROM DOCS WHERE USERS.ID=DOCS.USER_ID AND USERS.ID>1) ORDER BY NAMEUSER";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			return rs;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	public static boolean hasDocs(String iduser)
	{
		Statement stat = null;
		ResultSet rs = null;

		String sql = "	SELECT * FROM USERS WHERE EXISTS (SELECT * FROM DOCS WHERE USERS.ID=DOCS.USER_ID AND USERS.ID>1) AND USERS.ID=" + iduser + " ORDER BY NAMEUSER";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next())
			{
				rs.close();
				//Connection temp = stat.getConnection();
				//stat.close();
				//temp.close();
				return true;
			}
			else
			{
				rs.close();
				//Connection temp = stat.getConnection();
				//stat.close();
				//temp.close();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return false;
	}

	/**
	 * Me devuelve un Resultset con los usuarios de la aplicación
	 * 
	 * @return
	 */
	public static ResultSet getUserList()
	{
		ResultSet rs = userList();
		return rs;
	}

	/**
	 * Me devuelve un Resultset con los documentos de un usuario
	 * 
	 * @param iduser
	 *            Id del usuario
	 * @return
	 */
	public static ResultSet getUserDocs(String iduser)
	{
		Statement stat = null;
		ResultSet rs = null;

		String sql = "SELECT ID,TITLE FROM DOCS ORDER BY TITLE WHERE USER_ID=" + iduser;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			return rs;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * 
	 * Crea un nuevo usuario y toda su estructura de ficheros
	 * 
	 * @param name
	 */
	public static void newUserDBandProfile(String name)
	{
		int id = newUserDB(name);
		createUserProfile(id);
	}

	public static void createUserProfile(int id)
	{
		String so = Global.USERS_FOLDER + File.separator + String.valueOf(Global.USER_DEFAULT_TEMP_ID);
		String sd = Global.USERS_FOLDER + File.separator + String.valueOf(id);

		copyDirectory(new File(so), new File(sd));
	}

	/**
	 * 
	 * Array de JButtons, cada usuario tiene un boton
	 * 
	 * @param btnwidth
	 *            Ancho de cada botón
	 * @param btnheight
	 *            Alto de cada botón
	 * @return Array de JButtons
	 */
	public static JButton[] getUsersButtons(int btnwidth, int btnheight)
	{
		JButton[] j = null;
		j = getUsersButtons(btnwidth, btnheight, 1); // Por defecto sólo quiero los activos
		return j;
	}

	/**
	 * 
	 * Array de JButtons, cada usuario tiene un boton
	 * 
	 * @param btnwidth
	 *            Ancho de cada botón
	 * @param btnheight
	 *            Alto de cada botón
	 * @param state
	 *            Estado
	 * @return Array de JButtons
	 */
	public static JButton[] getUsersButtons(int btnwidth, int btnheight, int state)
	{
		int nusers;
		int i = 0;

		nusers = DataUsers.countUsers();
		ResultSet users = null;
		JButton[] tempBotones = new JButton[nusers];

		users = DataUsers.userList();
		try
		{
			if(users != null)
			{
				while (users.next())
				{ // Si quiero todos los usuarios o el usuario esta a 1
					if(state == 0 || users.getInt("ENABLE") == 1)
					{
						JButton tempButton = new JButton();
						String ruta = Global.USERS_FOLDER + File.separator + users.getString("ID") + File.separator + Global.USER_AVATAR + Global.PICT_EXT_PUNTO;
						ImageIcon iPicto = new ImageIcon(ruta);
						Image img = iPicto.getImage();
						Image newimg = img.getScaledInstance(btnwidth, btnheight, java.awt.Image.SCALE_SMOOTH);
						ImageIcon newIcon = new ImageIcon(newimg);
						tempButton.setIcon(newIcon);
						//tempButton.setBorderPainted(false);
						//tempButton.setBackground(Color.WHITE);
						tempButton.setHorizontalTextPosition(JButton.CENTER);
						tempButton.setVerticalTextPosition(JButton.BOTTOM);
						tempButton.setText(users.getString("NAMEUSER"));
						tempButton.setActionCommand(users.getString("ID"));
						tempBotones[i] = tempButton;
						i++;
					}
				}

				users.close();

				return tempBotones;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * 
	 * Array de JButtons, cada usuario tiene un boton
	 * 
	 * @param btnwidth
	 *            Ancho de cada botón
	 * @param btnheight
	 *            Alto de cada botón
	 * @param state
	 *            Estado
	 * @return Array de JButtons
	 */
	public static JButton getLastUserButton(int btnwidth, int btnheight, int state)
	{
		String[] user = null; // user[0] user[1]

		user = DataUsers.getLastUser();
		try
		{
			JButton tempButton = new JButton();
			String ruta = Global.USERS_FOLDER + File.separator + user[0] + File.separator + Global.USER_AVATAR + Global.PICT_EXT_PUNTO;
			ImageIcon iPicto = new ImageIcon(ruta);
			Image img = iPicto.getImage();
			Image newimg = img.getScaledInstance(btnwidth, btnheight, java.awt.Image.SCALE_SMOOTH);
			ImageIcon newIcon = new ImageIcon(newimg);
			tempButton.setIcon(newIcon);
			tempButton.setBorderPainted(false);
			tempButton.setBackground(Color.WHITE);
			tempButton.setHorizontalTextPosition(JButton.CENTER);
			tempButton.setVerticalTextPosition(JButton.BOTTOM);
			tempButton.setText(user[1]); // Nombre del usuario
			tempButton.setActionCommand(user[0]);

			return tempButton;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * 
	 * Copia un directorio
	 * 
	 * @param srcDir
	 *            Directorio de origen
	 * @param dstDir
	 *            Directorio de destino
	 */
	public static void copyDirectory(File srcDir, File dstDir)
	{
		if(srcDir.isDirectory())
		{
			String pc = String.valueOf(srcDir.getName().charAt(0));
			if(!pc.equals(".")) // Primer caracter
			{
				if(!dstDir.exists())
				{
					dstDir.mkdir();
				}

				String[] children = srcDir.list();
				for (int i = 0; i < children.length; i++)
				{
					copyDirectory(new File(srcDir, children[i]), new File(dstDir, children[i]));
				}
			}
		}
		else
		{
			copyFile(srcDir, dstDir);
		}
	}

	/**
	 * 
	 * Copia un fichero
	 * 
	 * @param src
	 *            fichero de origen
	 * @param dst
	 *            fichero de destino
	 */
	public static void copyFile(File src, File dst)
	{
		String pc = String.valueOf(src.getName().charAt(0));
		if(!pc.equals(".")) // Primer caracter
		{
			try
			{
				InputStream in = new FileInputStream(src);
				OutputStream out = new FileOutputStream(dst);

				byte[] buf = new byte[1024];
				int len;
				while ((len = in.read(buf)) > 0)
				{
					out.write(buf, 0, len);
				}
				in.close();
				out.close();
			}
			catch (Exception e)
			{
				System.out.println(e);
			}
		}
	}

	 /**
     * Reads a file as a single string.
     *
     * @param file
     *          The file to be turned into a String
     * @return  The file as String encoded in the platform
     * default encoding
     * 
     * @author Charles (http://technojeeves.com/joomla/index.php/free/93-file-to-string-in-java)
     */
    public static String fileToString(String file) {
        String result = null;
        DataInputStream in = null;

        try {
            File f = new File(file);
            byte[] buffer = new byte[(int) f.length()];
            in = new DataInputStream(new FileInputStream(f));
            in.readFully(buffer);
            result = new String(buffer);
        } catch (Exception e) {
            throw new RuntimeException("IO problem in fileToString", e);
        } finally {
            try {
                in.close();
            } catch (Exception e) { /* ignore it */
            }
        }
        return result;
    }
	
	public static boolean isEmptyFolder(String fpath)
	{
		return isEmptyFolder(fpath, null);
	}

	/**
	 * 
	 * Si una carpeta está vacia
	 * 
	 * @param fpath
	 *            ruta de la carpeta
	 * @param filter
	 *            filtro
	 * @return
	 */
	public static boolean isEmptyFolder(String fpath, FilenameFilter filter)
	{
		File dir = new File(fpath);
		String[] ficheros;

		if(filter == null)
		{
			ficheros = dir.list();
		}
		else
		{
			ficheros = dir.list(filter);
		}

		if(ficheros.length == 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * 
	 * Borra un documento de la BD y del sistema de ficheros
	 * 
	 * @param iddoc
	 *            id del documento
	 * @param iduser
	 *            id del usuario
	 */
	public static void deleteDocumentFileAndDB(String iddoc, String iduser)
	{
		String sql = null;
		Statement stat = null;

		sql = "DELETE FROM DOCS WHERE ID=" + iddoc;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			stat.execute(sql);
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}

		// Fichero
		String f = Global.USERS_FOLDER + File.separator + iduser + File.separator + iddoc + "." + Global.DOC_EXT;
		File fichero = new File(f);
		fichero.delete();

		// Previsualización
		f = Global.USERS_FOLDER + File.separator + iduser + File.separator + iddoc + "." + Global.PICT_EXT;
		fichero = new File(f);
		fichero.delete();
	}

	/**
	 * Añade un documento a la BD
	 * 
	 * @param title
	 *            Título del d
	 * @param pict
	 * @param id_user
	 * @return
	 */
	public static String insertDocumentFileAndDB(String filepath, String title, String iduser)
	{
		String iddoc = insertDocumentDB(title, Global.DEFAULT_PICT, iduser);
		String rdest = Global.USERS_FOLDER + File.separator + iduser + File.separator + iddoc + "." + Global.DOC_EXT;
		copyFile(new File(filepath), new File(rdest));

		return iddoc;
	}

	/**
	 * Delvuelve un vector con el nombre de todos los ficheros de un directorio
	 * 
	 * @param fpath
	 *            ruta al directorio
	 * 
	 */
	public static Vector<String> getFolderFileList(String fpath)
	{
		Vector<String> temp;
		temp = getFolderFileList(fpath, true);
		return temp;
	}

	/**
	 * Delvuelve un vector con el nombre de todos los ficheros de un directorio
	 * 
	 * @param fpath
	 *            ruta al directorio
	 * @param showext
	 *            muesta o nó las extensiones de los ficheros
	 * @return
	 */
	public static Vector<String> getFolderFileList(String fpath, Boolean showext)
	{
		FileFilterTCV filtro = new FileFilterTCV();
		File dir = new File(fpath);
		String[] ficheros = dir.list(filtro);
		Vector<String> temp = new Vector<String>();
		if(ficheros == null)
		{
			return null;
		}
		else
		{
			if(showext)
			{
				for (int i = 0; i < ficheros.length; i++)
				{
					temp.add(ficheros[i]);
				}
			}
			else
			// Sin extensión
			{
				for (int i = 0; i < ficheros.length; i++)
				{
					temp.add(ficheros[i].split("\\.")[0]);
				}
			}

			return temp;
		}
	}

	/**
	 * Inserta un documento en la BD
	 * 
	 * @param tile
	 *            Título del documento
	 * @param img
	 *            Imagen del documento
	 * @param iduser
	 *            Id del usuario
	 * @return Devuelve el id del nuevo documento
	 */
	public static String insertDocumentDB(String title, String img, String iduser)
	{
		String sql = null, ndoc = null;
		String fecha, dia, mes, anno, hora, minuto, segundo;
		Statement stat = null;

		Calendar cal = Calendar.getInstance();

		while (existDocumentTitle(iduser, title)) // Si el nombre del documento ya existe
		{
			title += "_copia";
		}

		dia = Integer.toString(cal.get(Calendar.DATE));
		if(dia.length() == 1) dia = "0" + dia;
		mes = Integer.toString(cal.get(Calendar.MONTH) + 1); // Mes 0 enero
		if(mes.length() == 1) mes = "0" + mes;
		anno = Integer.toString(cal.get(Calendar.YEAR));

		hora = Integer.toString(cal.get(Calendar.HOUR_OF_DAY));
		minuto = Integer.toString(cal.get(Calendar.MINUTE));
		segundo = Integer.toString(cal.get(Calendar.SECOND));

		if(hora.length() == 1) hora = "0" + hora;
		if(minuto.length() == 1) minuto = "0" + hora;
		if(segundo.length() == 1) segundo = "0" + hora;

		fecha = anno + "-" + mes + "-" + dia + " " + hora + ":" + minuto + ":" + segundo; // Formato
		// MySQL

		sql = "INSERT INTO DOCS(TITLE,NAMEDOC,DATE_CREATION,DATE_MODIFICATION,USER_ID,IMG_ID) " + "VALUES ('" + title + "','" + ndoc + "','" + fecha + "','" + fecha + "'," + iduser + "," + img + ")";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			stat.execute(sql);
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
			String id = getLastDocumentId(iduser);
			return id;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Devuelve el id del último documento de un usuario
	 * 
	 * @param id_user
	 *            identificador de usuario
	 * @return id id del último documento de un usuario
	 */
	public static String getLastDocumentId(String iduser)
	{
		String sql = null;
		String id = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT MAX(ID) AS ID FROM DOCS WHERE USER_ID=" + iduser;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			id = rs.getString("ID");
			//Connection temp = stat.getConnection();
			rs.close();
			//stat.close();
			//temp.close();
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}

		return id;
	}

	/**
	 * Devuelve un recordset con la información del último documento creado por un usuario
	 * 
	 * @param id_user
	 *            identificador de usuario
	 * @return RecordSet con la información del último documento creado por un usuario
	 */
	public static String[] getLastDocument(String id_user)
	{
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		String[] resultado = new String[2];

		sql = "SELECT * FROM DOCS WHERE USER_ID=" + id_user + " ORDER BY DATE_MODIFICATION DESC,ID DESC LIMIT 1";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next())
			{
				resultado[0] = rs.getString("ID");
				resultado[1] = rs.getString("TITLE");
			}
			else
			// No tiene documentos
			{
				resultado = null;
			}
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			resultado = null;
		}

		return resultado;
	}

	/**
	 * Devuelve true si un documento existe
	 * 
	 * @param id_user
	 *            id del usuario
	 * @param docname
	 *            título del documento
	 * @return true si existe false si no existe
	 */
	public static boolean existDocumentTitle(String id_user, String docname)
	{
		boolean exist = false;
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT ID FROM DOCS WHERE USER_ID=" + id_user + " AND TITLE='" + docname + "' COLLATE NOCASE";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next()) exist = true;
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return exist;
	}

	public static boolean existUserName(String name)
	{
		boolean exist = false;
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT NAMEUSER FROM USERS WHERE NAMEUSER='" + name + "'COLLATE NOCASE";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next()) exist = true;
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return exist;
	}

	/**
	 * Devuelve un vector de array de cadenas con toda la información de los documentos
	 * 
	 * * @param id_user id del usuario
	 * 
	 * @return
	 */
	public static Vector<String> getUserDocList(String id_user)
	{
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		String[] resultado = new String[2];
		Vector<String> ve = new Vector<String>();

		sql = "SELECT ID,TITLE FROM DOCS WHERE USER_ID=" + id_user + " ORDER BY DATE_MODIFICATION DESC";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			while (rs.next()) // Si existe el usuario en la BD
			{
				String id = rs.getString("ID");
				String title = rs.getString("TITLE");
				String elem = id + " - " + title;
				ve.add(elem);
			}
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
			return ve;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Devuelve un vector de array de cadenas con toda la información de los documentos
	 * 
	 * * @param id_user id del usuario
	 * 
	 * @return
	 */
	public static Vector<String[]> getUserDocListAll(String id_user)
	{
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		Vector<String[]> ve = new Vector<String[]>();

		sql = "SELECT ID,TITLE,DATE_CREATION,DATE_MODIFICATION FROM DOCS WHERE USER_ID=" + id_user + " ORDER BY DATE_MODIFICATION DESC";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			while (rs.next()) // Si existe el usuario en la BD
			{
				String[] dato = new String[4];
				dato[0] = rs.getString("ID");
				dato[1] = rs.getString("TITLE");
				dato[2] = rs.getString("DATE_CREATION");
				dato[3] = rs.getString("DATE_MODIFICATION");

				ve.add(dato);
			}

			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
			return ve;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Los últimos documentos editados por el usuario
	 * 
	 * @param id_user
	 *            identificador del usuario
	 * @return RecordSet con la información de los últimos documentos editados por el usuario
	 */
	public static ResultSet getLastDocuments(String id_user)
	{
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		String limit = "200";

		sql = "SELECT * FROM DOCS WHERE USER_ID=" + id_user + " ORDER BY DATE_MODIFICATION DESC LIMIT " + limit;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return rs;
	}

	/**
	 * Devuelve un resulset con los documentos de un usuario
	 * 
	 * @param id_user
	 *            identificador del usuario
	 * @param search
	 *            Cadena que debe que contener el nombre de los documentos
	 * @return resulset con los resultados
	 */
	public static ResultSet getLastDocuments(String id_user, String search)
	{
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		String limit = "20";

		sql = "SELECT * FROM DOCS WHERE USER_ID=" + id_user + " AND TITLE LIKE '%" + search + "%' ORDER BY DATE_MODIFICATION DESC LIMIT " + limit;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return rs;
	}

	/**
	 * Actualiza la fecha de modificación de un documento
	 * 
	 * @param id_document
	 *            Identificador del documento
	 */
	public static void setUpdateDateDocument(String id_document)
	{
		String sql, fecha, dia, mes, anno, hora, minuto, segundo;
		Statement stat = null;

		Calendar cal = Calendar.getInstance();

		dia = Integer.toString(cal.get(Calendar.DATE));
		mes = Integer.toString(cal.get(Calendar.MONTH) + 1); // Mes 0 enero
		anno = Integer.toString(cal.get(Calendar.YEAR));

		hora = Integer.toString(cal.get(Calendar.HOUR_OF_DAY));
		minuto = Integer.toString(cal.get(Calendar.MINUTE));
		segundo = Integer.toString(cal.get(Calendar.SECOND));

		if(hora.length() == 1) hora = "0" + hora;
		if(minuto.length() == 1) minuto = "0" + hora;
		if(segundo.length() == 1) segundo = "0" + hora;

		fecha = anno + "-" + mes + "-" + dia + " " + hora + ":" + minuto + ":" + segundo;

		sql = "UPDATE DOCS SET DATE_MODIFICATION='" + fecha + "' where ID=" + id_document;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			stat.execute(sql);
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Comprueba si el nombre de usuario y password son válidos, en caso de la información sea correcta devuelve un array con toda la información si falla la autentificación devuelve un null.
	 * 
	 * @param name
	 *            Nombre del usuario
	 * @param password
	 *            Password del usuario
	 * @return devuelve null si el usuario no se puede autentificar y en caso de que si devuelve un array de String con toda la información de este ("ID" , "NAMEUSER" , "TYPEUSER" , "PASSUSERTEXT" ,
	 *         "CONFIGUSER")
	 * 
	 */
	public static String[] authUser(String name, String password)
	{
		String info[] = null;
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT * FROM USERS WHERE NAMEUSER='" + name + "' AND PASSUSERTEXT='" + password + "'";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next()) // Si existe el usuario en la BD
			{
				info = new String[5];
				info[0] = rs.getString("ID");
				info[1] = rs.getString("NAMEUSER");
				info[2] = rs.getString("TYPEUSER");
				info[3] = rs.getString("PASSUSERTEXT");
				info[4] = rs.getString("CONFIGUSER");
			}
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return info;
	}

	/**
	 * Devuelve toda la información de usuario, si el usuario no existe devuelve null
	 * 
	 * @param name
	 *            Nombre del usuario
	 * @return devuelve null si el usuario no existe y si existe devuelve un array de String con toda la información de este ("ID" , "NAMEUSER" , "TYPEUSER" , "PASSUSERTEXT" , "CONFIGUSER")
	 * 
	 */
	public static String[] getUser(String name)
	{
		String info[] = null;
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT * FROM USERS WHERE NAMEUSER='" + name + "'";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next()) // Si existe el usuario en la BD
			{
				info = new String[5];
				info[0] = rs.getString("ID");
				info[1] = rs.getString("NAMEUSER");
				info[2] = rs.getString("TYPEUSER");
				info[3] = rs.getString("PASSUSER");
				info[4] = rs.getString("CONFIGUSER");
			}
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return info;
	}

	public static String[] getUser(int id)
	{
		String info[] = null;
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT * FROM USERS WHERE ID=" + id;

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			if(rs.next()) // Si existe el usuario en la BD
			{
				info = new String[5];
				info[0] = rs.getString("ID");
				info[1] = rs.getString("NAMEUSER");
				info[2] = rs.getString("TYPEUSER");
				info[3] = rs.getString("PASSUSER");
				info[4] = rs.getString("CONFIGUSER");
			}
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return info;
	}

	public static String[] getLastUser()
	{
		String[] info = new String[2];
		String sql = null;
		Statement stat = null;
		ResultSet rs = null;

		sql = "SELECT MAX(ID) AS ID,NAMEUSER FROM USERS";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			info[0] = rs.getString("ID");
			info[1] = rs.getString("NAMEUSER");
			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return info;
	}

	public static boolean isTICaDocFile(String file)
	{
		String[] ff = file.split("\\.");

		if(ff.length > 1)
		{
			String ext = ff[ff.length - 1];
			if(ext.equals(Global.DOC_EXT)) return true;
		}

		return false;
	}

	public static void writeUserName(int userid)
	{
		String rutaFichero = Global.USERS_FOLDER + File.separator + String.valueOf(userid) + File.separator + Global.USER_FILE_NAME;
		String[] info = DataUsers.getUser(userid);

		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(rutaFichero));
			out.write(info[1]); // Escribe el nombre del usuario
			out.close();
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}

	public static String readUserName(int userid)
	{
		String rutaFichero = Global.USERS_FOLDER + File.separator + String.valueOf(userid) + File.separator + Global.USER_FILE_NAME;
		String name = null;
		File archivo = new File(rutaFichero);
		FileReader fr;

		try
		{
			fr = new FileReader(archivo);
			BufferedReader br = new BufferedReader(fr);
			name = br.readLine();
			br.close();
			fr.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return name.trim();
	}

	public static String readUserName(String filepath)
	{
		String rutaFichero = filepath + File.separator + Global.USER_FILE_NAME;
		String name = null;
		File archivo = new File(rutaFichero);
		FileReader fr;

		// System.out.println("Cosa: " + rutaFichero);

		try
		{
			fr = new FileReader(archivo);
			BufferedReader br = new BufferedReader(fr);
			name = br.readLine();
			br.close();
			fr.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return name.trim();
	}

	public static Integer newUserDB(String username)
	{
		Statement stat = null;

		String sql = "INSERT INTO USERS(NAMEUSER,ENABLE) VALUES ('" + username + "',1)";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			stat.execute(sql);
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		int id = Integer.parseInt(getLastUser()[0]);

		return id;
	}

	public static void setPassAdmin(String pass)
	{
		Statement stat = null;

		pass = MD5(pass); // Obtengo el MD5 del pass dado por el usuario

		String sql = "UPDATE USERS SET PASSUSER='" + pass + "' WHERE ID=1";

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			stat.execute(sql);
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public static void deleteDBPicto()
	{
		String sql;
		Statement stat = null;
		String tablas[] = { "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "noletter", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "ñ" };
		try
		{
			stat = Global.DB_HANDLER.getStatement();
		}
		catch (SQLException e1)
		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		for (int x = 0; x < tablas.length; x++)
		{
			sql = "DELETE FROM " + tablas[x];
			try
			{
				stat.execute(sql);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		/*try
		{
			//stat.close();
		}
		catch (SQLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
	}

	public static void deletePictFolder()
	{
		deleteDirectory(new File(Global.PICT_FOLDER));
		createFolder(Global.PICT_FOLDER);
	}

	public static String[] ripperFile(String filename)
	{

		String[] resultado = new String[2];
		String f[] = filename.split("\\.");
		if(f.length > 1)
		{
			for (int i = 0; i <= f.length - 2; i++)
			{
				resultado[0] = resultado[0] + f[i];
			}
			resultado[1] = resultado[f.length - 1];
		}
		else
		{
			resultado[0] = filename;
			resultado[1] = "";
		}

		return resultado;
	}

	public static void writeImportDocList(String iduser)
	{
		BufferedWriter out = null;
		String rutaFichero = Global.USERS_FOLDER + File.separator + String.valueOf(iduser) + File.separator + Global.USER_IMPORT_DOC_LIST;
		Vector<String[]> ldoc = null;
		String[] dato = null;

		ldoc = getUserDocListAll(iduser);

		/*
		 * System.out.println("Datos " + ldoc.get(0)[0]); System.out.println("Datos " + ldoc.get(1)[0]); System.out.println("Datos " + ldoc.get(2)[0]);
		 */

		try
		{
			out = new BufferedWriter(new FileWriter(rutaFichero));
			for (int i = 0; i < ldoc.size(); i++)
			{
				dato = ldoc.get(i);
				String linea = dato[0] + " | " + dato[1];
				out.write(linea); // Escribe el nombre del usuario
				out.newLine();
			}
			out.close();
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}

	public static void UpdatePicto(String fupdate, JButton boton, String gentext)
	{
		String dpath = Global.UPDATE_FOLDER;
		String updatePictFolder = Global.UPDATE_FOLDER + File.separator + Global.UPDATE_FOLDER_PICT;

		System.out.println(" = Comienza la actualización de los pictogramas =");
		System.out.println("1/7 Borro la carpeta temporal " + dpath);
		boton.setText(gentext + " (0%)");
		boton.paintImmediately(boton.getVisibleRect());
		deleteDirectory(new File(dpath)); // Borra el directorio temporal antes de empezar con todo el proceso de la act.
		boton.setText(gentext + " (15%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("2/7 Descomprimo el fichero" + fupdate + " -> " + dpath);
		utils.UnZip.unZipIt(fupdate, dpath); // Descomprimo el archivo con los pictogramas y fichero de inserts
		boton.setText(gentext + " (30%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("3/7 Borro los pictogramas originales");
		deletePictFolder(); // Borro todo los ficheros de pictogramas
		boton.setText(gentext + " (45%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("4/7 Copio los pictogramas " + updatePictFolder + " -> " + Global.PICT_FOLDER);
		copyDirectory(new File(updatePictFolder), new File(Global.PICT_FOLDER)); // Copio los nuevos ficheros de pictogramas
		boton.setText(gentext + " (60%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("5/7 Borro las tablas de pictogramas");
		deleteDBPicto(); // Vacio las tablas de pictogramas
		boton.setText(gentext + " (75%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("6/7 Insert de los nuevos pictogramas");
		updateDBPicto(); // Añado los nuevos pictogramas a la BD
		boton.setText(gentext + " (100%)");
		boton.paintImmediately(boton.getVisibleRect());
		System.out.println("7/7 Fin de la actualización");
	}

	public static int checkPicto()
	{
		int err = 0;

		String sql = null;
		Statement stat = null;
		ResultSet rs = null;
		String img = null;
		String tablas[] = { "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "noletter", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "ñ" };
		String ruta = null;
		File fichero = null;

		for (int x = 0; x < tablas.length; x++)
		{
			if(x == 0) sql = " SELECT DISTINCT IMG_ID FROM (SELECT IMG_ID FROM " + tablas[x]; // Primero
			if(x > 0 && x != tablas.length - 1) sql = sql + " UNION SELECT IMG_ID FROM " + tablas[x];
			if(x == tablas.length - 1) sql = sql + " UNION SELECT IMG_ID FROM " + tablas[x] + ")"; // Último
		}

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			rs = stat.executeQuery(sql);
			while (rs.next())
			{
				img = rs.getString("IMG_ID");
				ruta = Global.PICT_FOLDER + File.separator + img + Global.PICT_EXT_PUNTO;
				fichero = new File(ruta);
				if(!fichero.exists())
				{
					err = err + 1; // Si el fichero no existe
				}
			}

			rs.close();
			//Connection temp = stat.getConnection();
			//stat.close();
			//temp.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return err;
	}

	public static void updateDBPicto()
	{
		String insertfile = Global.UPDATE_FOLDER + File.separator + Global.UPDATE_INSERTSQL;
		File file = new File(insertfile); // Fichero con los inserts
		BufferedReader bufRdr = null;
		Statement stat = null;
		// System.out.println(insertfile);

		try
		{
			stat = Global.DB_HANDLER.getStatement();
			bufRdr = new BufferedReader(new FileReader(file));
		}
		catch (Exception e)
		{
			System.out.println(e);
		}

		String line = null;

		try
		{
			while ((line = bufRdr.readLine()) != null) // Leo linea a linea
			{
				// System.out.println(line);
				stat.execute(line);
			}
			//stat.close();
		}
		catch (Exception e)
		{
			System.out.println(line);
			System.out.println(e);
		}
	}

	public static boolean isTICaImgFile(String file)
	{
		return true;
	}
}
