package virtual;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;

import utils.Global;
import editor.ConnectDB;
import editor.Editor;
import gui.ImageArray;


/**
 * 
 * @author njerez, hqcasanova
 */
public class Main {
		
	/*
     *  FUNCIÓN setPanelFrame
     *  @author njerez
     */
    public Main() {
    	final JFrame jF1;  	// jF1=Frame principal
    	JPanel jP1;  		// jP1=Contiene los botones    	       	 
    	JLabel jL1; 		// jL1=Etiqueta con título e imagen
    	JButton jBTV1, 		// jBTV1=Botón con acceso a TCV1
    	        jBTV2;  	// jBTV2=Botón con acceso a TCV2
    	
    	// jF1- Frame que contiene al Panel jP2
        jF1 = new JFrame();
        jF1.setResizable(false);
        try {
			jF1.setIconImage (ImageIO.read (new File (Global.FAVICON_TV)));
		} catch (IOException e) {
				System.out.println ("No se puede abrir icono.");
				e.printStackTrace();
		}              
    	
		// jP1- Panel con los botones
    	jP1 = new JPanel();
    	jP1.setPreferredSize (new Dimension (640, 480));
    	jP1.setLayout (new BoxLayout (jP1, BoxLayout.PAGE_AXIS));
    	jP1.setBorder (BorderFactory.createCompoundBorder (
    			                        BorderFactory.createLineBorder (new Color(205,218,232), 50), 
    			                        BorderFactory.createEtchedBorder (EtchedBorder.RAISED)));
    	jP1.setBackground(Color.WHITE);
    			
    	// Etiqueta con título
    	ImageArray imagen = new ImageArray (Global.BANNER, Global.IMG_FOLDER + Global.SEPARADOR_SYS, Global.PICT_EXT);
		imagen.resize (640, -1);
		jL1 = new JLabel (imagen.toIcon());
		jL1.setHorizontalTextPosition (JLabel.CENTER);
		jL1.setVerticalTextPosition (JLabel.BOTTOM);
		jL1.setAlignmentX (Component.CENTER_ALIGNMENT);
		jL1.setForeground (Global.DEFAULT_HIGHLIGHT);
    	jP1.add (jL1);    	    	   	
    	
    	// Fija localización y muestra ventana simple
    	jF1.getContentPane().add (jP1);
    	jF1.pack();
    	jF1.setLocationRelativeTo(null); // centrado
    	jF1.setVisible(true);
    	Editor.selectLanguage (jF1, null);
    	Global.DB_HANDLER = new ConnectDB (Global.INSTALL_FOLDER + Global.SEPARADOR_SYS + 
                                           Global.DB_FILE);
    	jL1.setText (Global.LANG.getString("tecvirt"));
    	jF1.setTitle (Global.LANG.getString("tecvirt"));
    	
    	// Botón para teclado virtual 1
        ImageIcon iTV1 = new ImageIcon(Global.RUTA_IMG_TCV1);      
        jBTV1 = new JButton (Global.LANG.getString("tipotcv1"), iTV1);
        jBTV1.setFont(Global.INTERFACE_FONT);
        jBTV1.setOpaque(true);
        jBTV1.setForeground(Color.GRAY);
    	jBTV1.setFocusable(false);
    	jBTV1.setBackground(Color.WHITE);
    	jBTV1.setCursor(Cursor.getPredefinedCursor (Cursor.HAND_CURSOR));
    	jBTV1.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	jF1.setVisible(false);
            	virtual.TecladoVirtual1 t = new TecladoVirtual1(jF1);
               	t.setVisible(true);
            }
    	});
    	jBTV1.setAlignmentX (Component.CENTER_ALIGNMENT);
    	jP1.add (Box.createRigidArea(new Dimension(0,55)));
    	jP1.add(jBTV1);
    	

    	// Botón para teclado virtual 2
    	ImageIcon iTV2 = new ImageIcon(Global.RUTA_IMG_TCV2);      
        jBTV2 = new JButton(Global.LANG.getString("tipotcv2"), iTV2);
        jBTV2.setFont(Global.INTERFACE_FONT);
        jBTV2.setOpaque(true);
        jBTV2.setForeground(Color.GRAY);
       	jBTV2.setFocusable(false);
    	jBTV2.setBackground(Color.WHITE);
    	jBTV2.setCursor(Cursor.getPredefinedCursor (Cursor.HAND_CURSOR));
    	jBTV2.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	jF1.setVisible(false);
             	virtual.TecladoVirtual2 t = new TecladoVirtual2(jF1);
             	t.setVisible(true);             	
            }
    	});
    	jBTV2.setAlignmentX (Component.CENTER_ALIGNMENT);
    	jP1.add (Box.createRigidArea(new Dimension(0,55)));
    	jP1.add(jBTV2);    	
    	    	 
    	jP1.repaint();
    	
    	
    }
	
	public static void main (String[] args) {
		UIManager.put ("ToolTip.background", Global.TOOLTIP_BACKGROUND);
		UIManager.put ("List.selectionBackground", Global.DEFAULT_HIGHLIGHT);
				
		try {
		    UIManager.setLookAndFeel (UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		        //Just load Java's native Look & Feel
		}
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
            	new Main();
            }
        });
    }
    
    
}
